print("import: 'terracotta'")
import terracotta

print("import: 'terracotta.client'")
import terracotta.client

print("import: 'terracotta.cmaps'")
import terracotta.cmaps

print("import: 'terracotta.drivers'")
import terracotta.drivers

print("import: 'terracotta.handlers'")
import terracotta.handlers

print("import: 'terracotta.scripts'")
import terracotta.scripts

print("import: 'terracotta.server'")
import terracotta.server

